import logging
import time
from threading import *
from AudioSource import *
import baco
from barix.media.BacoPlayer import BacoPlayer

class RtpAudioSource(AudioSource):
    def __init__(self, sourceId, name, volume, parameters, audioDev):
        AudioSource.__init__(self, sourceId, name, volume, parameters, audioDev)

        self.audioDevice = AudioSource.getAudioDev(self)
        self.url = AudioSource.getParameter(self, "url")
        self.url = self.url.replace("@","0.0.0.0")
        self.lock = Lock()

        self.monitoringThread = None
        self.isMonitoring = False

        self.logger = logging.getLogger('ipspeaker')

        self.externalCallback = None
        self.player = None

        self.playbackMode = AudioSource.getParameter(self, "playback_mode")
        if self.playbackMode == 'regular':
            self.logger.info("RTP Regular Mode")
            self.player = baco.SimplePlayer(self.url, self.audioDevice, 44100, baco.CONSTDELAY)
            self.player.setInputFifoDuration(0)
            self.player.setOutputFifoDuration(5000)
            self.player.setReadyThreshold(4500)
            self.player.setMaxDelta(30)

        elif self.playbackMode == 'low_latency':
            self.logger.info("RTP Low Latency Mode")
            self.player = BacoPlayer(self.url, self.audioDevice,44100)
        else:
            self.logger.error("RTP Playback Mode ERROR!")

        self.setEqualizer()
        self.player.setCallback(self.checkStateCallback)

    def connect(self):
        with self.lock:
            #self.logger.info(f"Connecting source id={self.id}, name={self.name}")
            if self.player is not None:
                if self.playbackMode == 'regular':
                    self.player.connectSource()
                else:
                    self.player.start()

    def disconnect(self):
        with self.lock:
            if self.player is not None:
                self.logger.info(f"Disconnecting source id={self.id}, name={self.name}")
                self.player.disconnect()

    def shutdown(self):
        self.logger.info("Shutdown RTP audio source")
        self.disconnect()

    def playAudio(self):
        with self.lock:
            if self.player is not None:
                self.logger.info(f"Playing source id={self.id}, name={self.name}")
                self.player.connectAudio()

    def stopAudio(self):
        with self.lock:
            if self.player is not None:
                self.logger.info(f"Muting source id={self.id}, name={self.name}")
                self.player.disconnectAudio()

    def reportStatus(self):
        sourceDict = {}
        sourceDict["id"] = self.id
        return sourceDict

    def setCallback(self, callback):
        self.externalCallback = callback

    def checkStateCallback(self, alive):
        self.logger.debug(f"Source {self.id}, alive {alive}")
        if self.externalCallback is not None:
            self.externalCallback(self, alive)

    def setEqualizer(self):
        with self.lock:
            AudioSource.setEqualizer(self, self.player)

    def setEqualizerParameter(self, config):
        with self.lock:
            return AudioSource.setEqualizerParameter(self, self.player, config)

    def enableEqualizer(self, enable):
        with self.lock:
            AudioSource.enableEqualizer(self, self.player, enable)
