import logging
import time
from threading import *
from AudioSource import *
import baco
from barix.media.BacoPlayer import BacoPlayer

class UdpAudioSource(AudioSource):
    def __init__(self, sourceId, name, volume, parameters, audioDev):
        AudioSource.__init__(self, sourceId, name, volume, parameters, audioDev)

        self.audioDevice = AudioSource.getAudioDev(self)
        self.url = AudioSource.getParameter(self, "url")
        self.url = self.url.replace("@","0.0.0.0")
        self.streamSampleRate = str(AudioSource.getParameter(self, "sample_rate"))
        self.streamFormat = str(AudioSource.getParameter(self, "audio_format"))
        self.lock = Lock()

        self.monitoringThread = None
        self.isMonitoring = False

        self.logger = logging.getLogger('ipspeaker')

        self.externalCallback = None
        self.player = BacoPlayer(self.url, self.audioDevice,44100)
        self.player.setInputFormat(self.streamFormat,self.streamSampleRate,"1")
        self.setEqualizer()
        self.player.setCallback(self.checkStateCallback)

    def connect(self):
        with self.lock:
            self.logger.info(f"Connecting source id={self.id}, name={self.name}")
            if self.player is not None:
                self.player.start()

    def disconnect(self):
        with self.lock:
            if self.player is not None:
                self.logger.info(f"Disconnecting source id={self.id}, name={self.name}")
                self.player.disconnect()

    def shutdown(self):
        self.logger.info("Shutdown RTP audio source")
        self.disconnect()

    def playAudio(self):
        with self.lock:
            if self.player is not None:
                self.logger.info(f"Playing source id={self.id}, name={self.name}")
                self.player.connectAudio()

    def stopAudio(self):
        with self.lock:
            if self.player is not None:
                self.logger.info(f"Muting source id={self.id}, name={self.name}")
                self.player.disconnectAudio()

    def reportStatus(self):
        sourceDict = {}
        sourceDict["id"] = self.id
        return sourceDict

    def setCallback(self, callback):
        self.externalCallback = callback

    def checkStateCallback(self, alive):
        self.logger.debug(f"Source {self.id}, alive {alive}")
        if self.externalCallback is not None:
            self.externalCallback(self, alive)

    def setEqualizer(self):
        with self.lock:
            AudioSource.setEqualizer(self, self.player)

    def setEqualizerParameter(self, config):
        with self.lock:
            return AudioSource.setEqualizerParameter(self, self.player, config)

    def enableEqualizer(self, enable):
        with self.lock:
            AudioSource.enableEqualizer(self, self.player, enable)
