import json
import logging
from flask import Blueprint, Response, request

from barix.web.auth import login_required

from scheduler import getSchedulerConfig, setSchedulerConfig, restartScheduler

log = logging.getLogger('flask-backend')

bp = Blueprint('scheduler_API', __name__)

@bp.route('/api/rest/v1/ipformer/scheduler', methods=['GET'])
@login_required
def handleGetSchedulerConfig():
    try:
        schedulerConfig = getSchedulerConfig()
    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        log.debug("scheduler config: {}".format(json.dumps(schedulerConfig)))
        return Response(json.dumps(schedulerConfig), status=200)

@bp.route('/api/rest/v1/ipformer/scheduler', methods=['PUT'])
@login_required
def handleSetSchedulerConfig():
    try:
        schedulerConfig = json.loads(request.data)
    except Exception as e:
        log.error(e)
        return Response("Bad formatted request body", status=400)
    try:
        log.info("Setting scheduler configuration: {}".format(schedulerConfig))
        setSchedulerConfig(schedulerConfig)
        restartScheduler()
    except Exception as e:
        log.error(e)
        return Response(status=500)
    else:
        return Response(status=200)


