//
// Created by to on 17/02/21.
//

#include <baco/ClockMonotonic.h>

#include "InputBlock.h"


using Poco::Logger;
using namespace baco;
using namespace std;

InputBlock::InputBlock(InputBlockConfig const& cfg, baco::FrameFormat const& outFmt) :
logger{ Logger::get("input") } ,
myConfig{cfg} ,
myInFmt{ ChannelLayout::STEREO, SampleFormat::I16, 2 , cfg.sampleRate },
myAlsaInput{ make_shared<AlsaFrameSource>(cfg.deviceName, myInFmt, cfg.bufferSize, cfg.periodSize) } ,
myGain{ make_shared<AudioGain>(myInFmt) } ,
myResampler{ make_shared<AudioResampler>(outFmt) } ,
myFifo{ make_shared<AudioFifo>(outFmt, cfg.fifoDelay * 1024) }
{
    myAlsaInput->setFrameSink(myGain);
    myGain->setFrameSink(myResampler);
    myResampler->setFrameSink(myFifo);

    myResampler->initialize(myInFmt);
    myFifo->setSilenceFeedMode(SilenceFeedMode::FORCE_OFF);
    if ( cfg.dispatchSize!=0 ) {
        myFifo->setDispatchFrameSize(cfg.dispatchSize);
    }
    myFifo->startDispatcher();
    myAlsaInput->open();
}

InputBlock::~InputBlock()
{
    myAlsaInput->close();
}

void InputBlock::attachSink(baco::FrameSinkPtr const& sink)
{
    myFifo->setFrameSink(sink);
}

baco::ReturnCode InputBlock::readFrame()
{
    int cnt = 0;
    auto r = myAlsaInput->readFrame(&cnt);
    myStats.totalSamplesIn += cnt;
    if ( r==ReturnCode::E_PIPE ) {
        myStats.xrunIn++;
        if (myStats.tFirstXrun==0 ) {
            myStats.tFirstXrun   = ClockMonotonic::getMillis();
        }
    }
    if ( cnt == 0) {
        return ReturnCode::E_BADSTATE;
    }
    int delay   = myFifo->getDelay(myInFmt.sampleRate());
    myLevelAcc += delay;
    myLevelCount++;
    myStats.fifoLevel   = (int)lround(myLevelAcc/myLevelCount);

    if ( delay>myStats.fifoLevelMax ) {
        myStats.fifoLevelMax    = delay;
        myStats.tLevelMax       = ClockMonotonic::getMillis();
    }
    else if ( delay<myStats.fifoLevelMin ) {
        myStats.fifoLevelMin    = delay;
        myStats.tLevelMin       = ClockMonotonic::getMillis();
    }
    return r;
}

int InputBlock::calculateCompensation()
{
    int delay           = myStats.fifoLevel;
    int compensation    = (myConfig.fifoDelay-delay)/4;
    log_information(logger,Poco::format("delay=%d target=%d comp=%d (frames)",delay,myConfig.fifoDelay,compensation));
    return compensation;
}

void InputBlock::applyCompensation(int comp)
{
    myStats.totalComp  += comp;
    myResampler->addCompensationFrames(comp);
}


void InputBlock::setGain(double gain, int durationMs, int delayMs)
{
    myGain->setGain(gain,durationMs,delayMs);
}


